<div class="modal fade" id="filtersModal" tabindex="-1" role="dialog" aria-labelledby="filters" aria-hidden="true">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <h4 class="modal-title">Filters</h4>
    </div>
      <div class="modal-body">
          {% for element in elements %}
          {% if not forloop.first %}<hr/>{% endif %}
          {{ element }}
          {% endfor %}
      </div>
    </div>
  </div>
</div>
