<div class="form-group">
  {% if field.label %}
    <label {% if style.hide_label %}class="sr-only"{% endif %}>{{ field.label }}</label>
  {% endif %}

  <p class="form-control-static">Dictionaries are not currently supported in HTML input.</p>
</div>
